@@ -10,7 +10,7 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
                        fixef, na_inf.rm = getFixest_na_inf.rm(), NL.start, lower, upper, NL.start.init,
                        offset, linear.start = 0, jacobian.method = "simple",
                        useHessian = TRUE, hessian.args = NULL, opt.control = list(),
-                      y, X, fixef_mat,
+                       y, X, fixef_mat, panel.id,
                        nthreads = getFixest_nthreads(),
                        verbose = 0, theta.init, fixef.tol = 1e-5, fixef.iter = 10000,
                        deriv.iter = 5000, deriv.tol = 1e-4, glm.iter = 25, glm.tol = 1e-8,
@@ -58,7 +58,7 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
 
     #
     # Arguments control
-    main_args = c("fml", "data", "offset", "na_inf.rm", "fixef.tol", "fixef.iter", "fixef", "nthreads", "verbose", "warn", "notes", "combine.quick", "start")
+    main_args = c("fml", "data", "panel.id", "offset", "na_inf.rm", "fixef.tol", "fixef.iter", "fixef", "nthreads", "verbose", "warn", "notes", "combine.quick", "start")
     femlm_args = c("family", "theta.init", "linear.start", "opt.control", "deriv.tol", "deriv.iter")
     feNmlm_args = c("NL.fml", "NL.start", "lower", "upper", "NL.start.init", "jacobian.method", "useHessian", "hessian.args")
     feglm_args = c("family", "weights", "glm.iter", "glm.tol", "etastart", "mustart")
@@ -199,8 +199,10 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
         # they concern only logit and poisson (so far)
         #
 
-        family$dev.resids = function(y, mu, eta, wt) sum(family$dev.resids(y, mu, wt))
-        family$mu.eta = function(mu, eta) family$mu.eta(eta)
+        fun_dev = family$dev.resids
+        family$dev.resids = function(y, mu, eta, wt) sum(fun_dev(y, mu, wt))
+        fun_mu.eta = family$mu.eta
+        family$mu.eta = function(mu, eta) fun_mu.eta(eta)
 
         if(is.null(family$valideta)) family$valideta = function(...) TRUE
         if(is.null(family$validmu)) family$validmu = function(...) TRUE
@@ -216,6 +218,9 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
         stop("Argument 'notes' must be a single logical.")
     }
 
+    show_notes = notes
+    notes = c()
+
     if(!isLogical(warn)){
         stop("Argument 'warn' must be a single logical.")
     }
@@ -260,6 +265,7 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
     # Formatting data ####
     #
 
+    isPanel = FALSE
     if(isFit){
         isFixef = !missnull(fixef_mat)
     } else {
@@ -273,6 +279,7 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
             }
             data = as.data.frame(data)
         }
+
         # The conversion of the data (due to data.table)
         if(!"data.frame" %in% class(data)){
             stop("The argument 'data' must be a data.frame or a matrix.")
@@ -291,6 +298,31 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
         fml = formula(fml) # we regularize the formula to check it
         if(length(fml) != 3) stop("The formula must be two sided: e.g. y~x1+x2, or y~x1+x2|fe1+fe2.")
 
+        #
+        # Panel setup
+        #
+
+        if(check_lag(fml)){
+            panel.info = NULL
+            isPanel = TRUE
+            if(!is.null(attr(data, "panel_info"))){
+                if(!missing(panel.id)){
+                    warning("The argument 'panel.id' is provided but argument 'data' is already a 'fixest_panel' object. Thus the argument 'panel.id' is ignored.", immediate. = TRUE)
+                }
+
+                panel__meta__info = attr(data, "panel_info")
+                panel.id = panel__meta__info$panel.id
+                panel.info = panel__meta__info$call
+            } else {
+                # Later: automatic deduction using the first two clusters
+                if(missing(panel.id)){
+                    stop("To use lag/leads (with l()/f()): either provide the argument 'panel.id' with the panel identifiers OR set your data as a panel with function panel().")
+                }
+                panel__meta__info = panel_setup(data, panel.id, from_fixest = TRUE)
+            }
+            class(data) = "data.frame"
+        }
+
         FML = Formula::Formula(fml)
         n_rhs = length(FML)[2]
 
@@ -298,6 +330,30 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
             stop("The argument 'fml' cannot contain more than two parts separated by a pipe ('|').")
         }
 
+        # We test there are no problems in the formula
+        info = try(terms(formula(FML, lhs = 1, rhs = 1)), silent = TRUE)
+        if("try-error" %in% class(info)){
+            dp = deparse(formula(FML, lhs = 1, rhs = 1))
+            stop("The formula: ", dp[1], ifsingle(dp, "", "..."), ", is not valid:\n", gsub("^[^\n]+\n", "", info))
+        }
+
+        # we check the FE part
+        if(n_rhs == 2){
+            info = terms_fixef(formula(FML, lhs = 0, rhs = 2))
+            if("try-error" %in% class(info)){
+                dp = deparse(formula(FML, lhs = 0, rhs = 2))
+                stop("The fixed-effects part of the formula: ", dp[1], ifsingle(dp, "", "..."), ", is not valid:\n", gsub("^[^\n]+\n", "", info))
+            }
+        }
+
+        if(isPanel){
+            fml = try(rewrite_fml(fml), silent = TRUE)
+            if("try-error" %in% class(fml)){
+                stop("Problem in the formula: ", gsub("_expand", "", fml))
+            }
+            FML = Formula::Formula(fml)
+        }
+
         # for clarity, arg fixef is transformed into fixef_vars
         fixef_vars = NULL
         isFixef = FALSE
@@ -309,7 +365,7 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
             } else {
                 stop("To add fixed-effects: either include them in argument 'fml' using a pipe ('|'), either use the argument 'fixef'. You cannot use both!")
             }
-        } else if(!missnull(fixef)){
+        } else if(!missing(fixef) && length(fixef) >= 1){
             isFixef = TRUE
 
             # We check the argument => character vector
@@ -425,14 +481,15 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
     # Controls and setting of the linear part:
     #
 
+    interaction.info = NULL
     if(isFit){
 
         isLinear = FALSE
-        if(is.null(X)){
+        if(missing(X) || is.null(X)){
             if(!isFixef){
                 stop("Argument X must be provided in the absence of fixed-effects.")
             } else {
-                isLinear = TRUE
+                isLinear = FALSE
             }
         } else {
             isLinear = TRUE
@@ -478,7 +535,18 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
 
     } else {
         isLinear = FALSE
-        linear.varnames = all.vars(fml[[3]])
+        options("fixest_interaction_ref" = NULL)
+
+        if(grepl("[^:]::[^:]", deparse_long(fml[[3]]))){
+
+            new_fml = try(interact_fml(fml), silent = TRUE)
+            if("try-error" %in% class(new_fml)){
+                stop("Error in the right-hand-side of the formula: ", new_fml)
+            }
+            linear.varnames = all.vars(new_fml[[3]])
+        } else {
+            linear.varnames = all.vars(fml[[3]])
+        }
 
         if(length(linear.varnames) > 0 || attr(terms(fml), "intercept") == 1){
             isLinear = TRUE
@@ -502,30 +570,40 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
             # We construct the linear matrix
             #
 
-            # we look at whether there are factor-like variables to be evaluated
-            # if there is factors => model.matrix
-            types = sapply(data[, dataNames %in% linear.varnames, FALSE], class)
-            if(length(types) == 0 || grepl("factor", deparse_long(linear.fml)) || any(types %in% c("character", "factor"))){
-                useModel.matrix = TRUE
-            } else {
-                useModel.matrix = FALSE
+            linear.mat = try(fixest_model_matrix(fml, data), silent = TRUE)
+            if("try-error" %in% class(linear.mat)){
+                stop("Evaluation of the right-hand-side of the formula raises an error: ", linear.mat)
             }
 
-            if(useModel.matrix){
-                # linear.mat = stats::model.matrix(linear.fml, data)
-                # to catch the NAs, model.frame needs to be used....
-                linear.mat = try(stats::model.matrix(linear.fml, stats::model.frame(linear.fml, data, na.action=na.pass)), silent = TRUE)
-                if("try-error" %in% class(linear.mat)){
-                    stop("Evaluation of the right-hand-side of the formula raises and error: \n", linear.mat)
-                }
-            } else {
-                # just to check => will give error if not proper formula
-                linear.mat = try(stats::model.matrix(linear.fml, head(data, 10)), silent = TRUE)
-                if("try-error" %in% class(linear.mat)){
-                    stop("Evaluation of the right-hand-side of the formula raises and error: \n", linear.mat)
-                }
-                linear.mat = prepare_matrix(linear.fml, data)
-            }
+            useModel.matrix = attr(linear.mat, "useModel.matrix")
+
+            # Interaction information => if no interaction: NULL
+            interaction.info = getOption("fixest_interaction_ref")
+
+
+            # # we look at whether there are factor-like variables to be evaluated
+            # # if there is factors => model.matrix
+            # types = sapply(data[, dataNames %in% linear.varnames, FALSE], class)
+            # if(length(types) == 0 || grepl("factor", deparse_long(linear.fml)) || any(types %in% c("character", "factor"))){
+            #     useModel.matrix = TRUE
+            # } else {
+            #     useModel.matrix = FALSE
+            # }
+            #
+            # if(useModel.matrix){
+            #     # linear.mat = stats::model.matrix(linear.fml, data)
+            #     # to catch the NAs, model.frame needs to be used....
+            #     linear.mat = try(stats::model.matrix(linear.fml, stats::model.frame(linear.fml, data, na.action=na.pass)), silent = TRUE)
+            #     if("try-error" %in% class(linear.mat)){
+            #         stop("Evaluation of the right-hand-side of the formula raises an error: \n", linear.mat)
+            #     }
+            # } else {
+            #     linear.mat = try(prepare_matrix(linear.fml, data), silent = TRUE)
+            #     if("try-error" %in% class(linear.mat)){
+            #         stop("Evaluation of the right-hand-side of the formula raises an error: \n", linear.mat)
+            #     }
+            # }
+
         }
 
     }
@@ -712,7 +790,7 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
             }
         } else if(is.null(offset)){
             # msg if it's not what the user wanted
-            if(!is.null(mc_origin$offset)){
+            if(!is.null(mc_origin$offset) && deparse_long(mc_origin$offset) != "x$offset"){
                 stop("Argument 'offset' (", deparse_long(mc_origin$offset), ") is evaluated to NULL. This is likely not what you want.")
             }
         }
@@ -810,10 +888,11 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
             if(any0W){
                 is0W = weights.value == 0
                 is0W = is0W & !is.na(is0W)
-                message_0W = paste0(numberFormatNormal(sum(is0W)), " observations removed because of 0-weight.")
+                message_0W = paste0(numberFormatNormal(sum(is0W)), " observation", plural(sum(is0W)), " removed because of 0-weight.")
+                notes = c(notes, message_0W)
             }
 
-        } else if(!is.null(mc_origin$weights)){
+        } else if(!is.null(mc_origin$weights) && deparse_long(mc_origin$weights) != 'x[["weights"]]'){
             # we avoid this behavior
             stop("Argument 'weights' (", deparse_long(mc_origin$weights), ") is evaluated to NULL. This is likely not what you want.")
         }
@@ -1011,10 +1090,18 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
             }
         }
 
-        # We change factors to character
-        isFactor = sapply(fixef_mat, is.factor)
-        if(any(isFactor)){
-            for(i in which(isFactor)){
+        # # We change factors to character
+        # isFactor = sapply(fixef_mat, is.factor)
+        # if(any(isFactor)){
+        #     for(i in which(isFactor)){
+        #         fixef_mat[[i]] = as.character(fixef_mat[[i]])
+        #     }
+        # }
+
+        # We change non-numeric to character (impotant for parallel qufing)
+        is_not_num = sapply(fixef_mat, function(x) !is.numeric(x))
+        if(any(is_not_num)){
+            for(i in which(is_not_num)){
                 fixef_mat[[i]] = as.character(fixef_mat[[i]])
             }
         }
@@ -1081,7 +1168,8 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
 
             if(anyNA_sample){
                 msg = msg_na_inf(ANY_NA, ANY_INF)
-                message_NA = paste0(numberFormatNormal(nbNA), " observations removed because of ", msg, " (Breakup: ", msgNA_y, msgNA_L, msgNA_NL, msgNA_cluster, msgNA_slope, msgNA_offset, msgNA_weight, ").")
+                message_NA = paste0(numberFormatNormal(nbNA), " observation", plural(nbNA), " removed because of ", msg, " (Breakup: ", msgNA_y, msgNA_L, msgNA_NL, msgNA_cluster, msgNA_slope, msgNA_offset, msgNA_weight, ").")
+                notes = c(notes, message_NA)
             }
 
             if(nbNA == nobs){
@@ -1115,154 +1203,102 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
 
             # we change the LHS variable
             lhs = lhs[-obs2remove_NA]
+
+            if(cpp_isConstant(lhs)){
+                # We absolutely need to control for that, otherwise, the code breaks later on
+
+                message(ifsingle(notes, "NOTE: ", "NOTES: "), paste(notes, collapse = "\n       "))
+                msg = "NAs"
+                if(any0W && !anyNA_sample) msg = "0-weight"
+                if(any0W && anyNA_sample) msg = "NAs and 0-weight"
+                stop("The dependent variable (after cleaning for ", msg, ") is a constant. Estimation cannot be done.")
+            }
         }
 
+        # QUF setup ####
+
         Q = length(fixef_terms) # terms: contains FEs + slopes
-        fixef_id = fixef_names = list()
-        sum_y_all = fixef_table = list()
-        obs2remove = fixef_sizes = c()
-        fixef_removed = list()
-        slope_variables = list()
-        for(i in 1:Q){
 
-            check_remove = TRUE
-            if(isSlope){
 
-                # the slope variable
-                if(slope_flag[i]){
-                    slope_variables[[i]] = slope_mat[[slope_vars[i]]]
-                    if(length(slope_variables[[i]]) == 1){
-                        # si l'utilisateur utilise une constante comme variable...
-                        # il faut aussi controler pour quand il fait n'importe quoi...
-                        slope_variables[[i]] = rep(slope_variables[[i]], length(lhs))
-                    }
-                }
 
-                if(i > 1 && slope_fe[i] %in% slope_fe[1:(i-1)]){
-                    # fe done already!
-                    i_done = which.max(slope_fe[1:(i-1)] == slope_fe[i])
-                    fixef_names[[i]] = fixef_names[[i_done]]
-                    fixef_id[[i]] = fixef_id[[i_done]]
-                    sum_y_all[[i]] = sum_y_all[[i_done]]
-                    fixef_table[[i]] = fixef_table[[i_done]]
-                    fixef_sizes[i] = fixef_sizes[i_done]
+        fixef_removed = slope_variables = list()
+        obs2remove = c()
 
-                    next
-                } else {
-                    dum_raw = fixef_mat[[slope_fe[i]]]
+        type = switch(family, gaussian = 0, logit = 2, 1)
+        do_sum_y = !origin_type %in% c("feols", "feglm")
 
-                    if(all(slope_flag)){
-                        # we check the removal of obs
-                        #    iff there is a "pure" fixef (ie no slope fixef)
-                        # condition means fixef id only associated to slopes
-                        check_remove = FALSE
-                    }
-
-                }
-            } else {
-                dum_raw = fixef_mat[[fixef_vars[i]]]
-            }
-
-            # FEs turned into integers
-            info = quickUnclassFactor(dum_raw, addItem = TRUE)
-            fixef_names[[i]] = thisNames = info$items
-            dum = info$x
-
-            fixef_id[[i]] = dum
-            k = length(thisNames)
-
-            # We delete "all zero" outcome
-            sum_y_all[[i]] = sum_y_clust = cpp_tapply_vsum(k, lhs, dum)
-            fixef_table[[i]] = n_perClust = cpp_table(k, dum)
-            fixef_sizes[i] = k
-
-            if(check_remove){
-                if(family %in% c("poisson", "negbin")){
-                    qui = which(sum_y_clust == 0)
-                } else if(family == "logit"){
-                    qui = which(sum_y_clust == 0 | sum_y_clust == n_perClust)
-                } else if(family == "gaussian"){
-                    qui = NULL
-                }
-            } else {
-                qui = NULL
-            }
-
-            # I don't do fixef_removed[[i]] = stg because of the slopes
-            # if no slope, this is identical to fixef_removed[[i]] = stg
-            # if slope: then length(fixef_removed) ends up being identical to length(fixef_vars)
-            #    (remember taht fixef_vars is the unique of slope_fe)
-            if(length(qui) > 0){
-                # We first delete the data:
-                # fixef_removed[[i]] = thisNames[qui]
-                fixef_removed[[length(fixef_removed) + 1]] = thisNames[qui]
-                obs2remove = unique(c(obs2remove, which(dum %in% qui)))
-            } else {
-                # fixef_removed[[i]] = character(0)
-                fixef_removed[[length(fixef_removed) + 1]] = character(0)
-            }
+        only_slope = FALSE
+        if(isSlope){
+            # only slope: not any non-slope
+            only_slope = as.vector(tapply(!slope_flag, slope_fe, sum)[fixef_vars]) == 0
         }
 
-        # We remove the problems
-        if(length(obs2remove) > 0){
+        quf_info_all = cpppar_quf_table_sum(x = fixef_mat, y = lhs, do_sum_y = do_sum_y, type = type, only_slope = only_slope, nthreads = nthreads)
+
+        fixef_id = quf_info_all$quf
+        # names
+        fixef_names = list()
+        is_string = sapply(fixef_mat, is.character)
+        for(i in 1:length(fixef_id)){
+            if(is_string[i]){
+                fixef_names[[i]] = fixef_mat[[i]][quf_info_all$items[[i]]]
+            } else {
+                fixef_names[[i]] = quf_info_all$items[[i]]
+            }
+        }
+        # table/sum_y/sizes
+        fixef_table = quf_info_all$table
+        sum_y_all = quf_info_all$sum_y
+        fixef_sizes = lengths(fixef_table)
+
+        # If observations have been removed:
+        if(!is.null(quf_info_all$obs_removed)){
+
+            # which obs are removed
+            obs2remove = which(quf_info_all$obs_removed)
 
             # update of the lhs
             lhs = lhs[-obs2remove]
 
-            # Then we recreate the dummies
-            for(i in 1:Q){
-
-                if(isSlope){
-
-                    # update of the slope variable
-                    if(slope_flag[i]){
-                        slope_variables[[i]] = slope_variables[[i]][-obs2remove]
-                    }
-
-                    if(i > 1 && slope_fe[i] %in% slope_fe[1:(i-1)]){
-                        # fe done already!
-                        i_done = which.max(slope_fe[1:(i-1)] == slope_fe[i])
-                        fixef_names[[i]] = fixef_names[[i_done]]
-                        fixef_id[[i]] = fixef_id[[i_done]]
-                        sum_y_all[[i]] = sum_y_all[[i_done]]
-                        fixef_table[[i]] = fixef_table[[i_done]]
-                        fixef_sizes[i] = fixef_sizes[i_done]
-
-                        next
-                    } else {
-                        dum_new = fixef_id[[i]][-obs2remove]
-                    }
+            # Names of the FE removed
+            for(i in 1:length(fixef_id)){
+                if(is_string[i]){
+                    fixef_removed[[i]] = fixef_mat[[i]][quf_info_all$fe_removed[[i]]]
                 } else {
-                    dum_new = fixef_id[[i]][-obs2remove]
+                    fixef_removed[[i]] = quf_info_all$fe_removed[[i]]
                 }
-
-                # new way: faster using cpp_update_dum
-                k = length(fixef_names[[i]])
-                info = cpp_update_dum(dum_new, k)
-                dum = info$dum_new
-                fixef_names[[i]] = fixef_names[[i]][info$keep == 1]
-
-                fixef_id[[i]] = dum
-                k = length(fixef_names[[i]])
-
-                # We also recreate these values
-                sum_y_all[[i]] = cpp_tapply_vsum(k, lhs, dum)
-                fixef_table[[i]] = cpp_table(k, dum)
-                fixef_sizes[i] = k
-
             }
 
-            # Then the "Notes"
-            nb_missing = sapply(fixef_removed, length)
-            message_cluster = paste0(paste0(nb_missing, collapse = "/"), " fixed-effect", ifelse(sum(nb_missing) == 1, "", "s"), " (", numberFormatNormal(length(obs2remove)), " observations) removed because of only ", ifelse(family=="logit", "zero (or only one)", "zero"), " outcomes.")
-
-            note = ifelse((anyNA_sample + any0W) > 0, "NOTES: ", "NOTE: ")
-            if(notes) message(note, message_NA, ifelse(anyNA_sample, "\n       ", ""), message_0W, ifelse(any0W, "\n       ", ""), message_cluster)
-
             names(fixef_removed) = fixef_vars
-        } else if(anyNA_sample){
-            note = ifelse((anyNA_sample + any0W) > 1, "NOTES: ", "NOTE: ")
-            if(notes) message(note, message_NA, ifelse(anyNA_sample && any0W, "\n       ", ""), message_0W)
+
+            # Then the "Notes"
+            nb_missing = lengths(fixef_removed)
+            message_cluster = paste0(paste0(nb_missing, collapse = "/"), " fixed-effect", plural(sum(nb_missing)), " (", numberFormatNormal(length(obs2remove)), " observation", plural_len(obs2remove), ") removed because of only ", ifelse(family=="logit", "zero (or only one)", "zero"), " outcomes.")
+            notes = c(notes, message_cluster)
+        }
+
+        # If slopes: we need to recreate some values (quf/table/sum_y)
+        if(isSlope){
+
+            # The slope variables
+            for(i in which(slope_flag)){
+                slope_variables[[i]] = slope_mat[[slope_vars[i]]]
+                if(length(slope_variables[[i]]) == 1){
+                    # si l'utilisateur utilise une constante comme variable...
+                    # il faut aussi controler pour quand il fait n'importe quoi...
+                    slope_variables[[i]] = rep(slope_variables[[i]], length(lhs))
+                }
+            }
+
+            dict = 1:length(fixef_vars)
+            names(dict) = fixef_vars
+            new_id = dict[slope_fe]
+
+            fixef_id = fixef_id[new_id]
+            fixef_names = fixef_names[new_id]
+            sum_y_all = sum_y_all[new_id]
+            fixef_table = fixef_table[new_id]
+            fixef_sizes = lengths(fixef_table)
         }
 
         if(length(obs2remove_NA) > 0){
@@ -1290,11 +1326,16 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
             index = 1:Q
         }
 
+
         fixef_id_res = list()
         for(i in index){
             dum = fixef_id[[i]]
             attr(dum, "fixef_names") = as.character(fixef_names[[i]])
-            fixef_id_res[[fixef_vars[i]]] = dum
+            if(isSlope){
+                fixef_id_res[[slope_fe[i]]] = dum
+            } else {
+                fixef_id_res[[fixef_vars[i]]] = dum
+            }
         }
 
         fixef_sizes_res = fixef_sizes[index]
@@ -1304,7 +1345,9 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
         # We re-order the clusters
         #
 
+        IS_REORDER = FALSE
         if(any(fixef_sizes != sort(fixef_sizes, decreasing = TRUE))){
+            IS_REORDER = TRUE
             # FE with the most cases first (limits precision problems)
 
             new_order = order(fixef_sizes, decreasing = TRUE)
@@ -1330,7 +1373,8 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
             nbNA = sum(isNA_sample)
 
             if(anyNA_sample){
-                message_NA = paste0(numberFormatNormal(nbNA), " observations removed because of NA values (Breakup: ", msgNA_y, msgNA_L, msgNA_NL, msgNA_offset, msgNA_weight, ").")
+                message_NA = paste0(numberFormatNormal(nbNA), " observation", plural(nbNA), " removed because of NA values (Breakup: ", msgNA_y, msgNA_L, msgNA_NL, msgNA_offset, msgNA_weight, ").")
+                notes = c(notes, message_NA)
 
                 if(nbNA == nobs){
                     stop("All observations contain NAs. Estimation cannot be done. (Breakup: ", msgNA_y, msgNA_L, msgNA_NL, msgNA_cluster, msgNA_offset, msgNA_weight, ")")
@@ -1351,8 +1395,8 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
                 }
             }
 
-            note = ifelse((anyNA_sample + any0W) == 2, "NOTES: ", "NOTE: ")
-            if(notes) message(note, message_NA, ifelse(anyNA_sample && any0W, "\n       ", ""), message_0W)
+            # note = ifelse((anyNA_sample + any0W) == 2, "NOTES: ", "NOTE: ")
+            # if(notes) message(note, message_NA, ifelse(anyNA_sample && any0W, "\n       ", ""), message_0W)
 
             # we drop the NAs from the fixef matrix
             obs2remove = which(isNA_sample)
@@ -1361,21 +1405,41 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
         }
     }
 
+    # Messages
+    if(show_notes && length(notes) > 0){
+        message(ifsingle(notes, "NOTE: ", "NOTES: "), paste(notes, collapse = "\n       "))
+    }
+
     # NA & problem management
     if(length(obs2remove) > 0){
         # we kick out the problems (both NA related and fixef related)
 
-        if(isNL || (isLinear && useModel.matrix)){
-            data = data[-obs2remove, , drop = FALSE]
-        }
+        # if(isNL || (isLinear && useModel.matrix)){
+        #     data = data[-obs2remove, , drop = FALSE]
+        # }
+        #
+        # # We recreate the linear matrix and the LHS
+        # if(isLinear) {
+        #     if(useModel.matrix){
+        #         # means there are factors
+        #         # linear.mat = stats::model.matrix(linear.fml, data)
+        #         linear.mat = fixest_model_matrix(linear.fml, data)
+        #     } else {
+        #         linear.mat = linear.mat[-obs2remove, , drop = FALSE]
+        #     }
+        # }
 
-        # We recreate the linear matrix and the LHS
-        if(isLinear) {
-            if(useModel.matrix){
-                # means there are factors
-                linear.mat = stats::model.matrix(linear.fml, data)
-            } else {
-                linear.mat = linear.mat[-obs2remove, , drop = FALSE]
+        # We drop only 0 variables (may happen for factors)
+        linear.mat = linear.mat[-obs2remove, , drop = FALSE]
+
+        if(useModel.matrix){
+            # There are factors => possibly some vars are only 0 now that NAs are removed
+
+            only_0 = cpppar_check_only_0(linear.mat, nrow(linear.mat), nthreads)
+            if(all(only_0 == 1)){
+                stop("After removing NAs, not a single explanatory variable is different from 0.")
+            } else if(any(only_0 == 1)){
+                linear.mat = linear.mat[, only_0 == 0, drop = FALSE]
             }
         }
 
@@ -1520,6 +1584,10 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
             assign("init.type", c("coef", "eta", "mu")[start_provided], env)
         }
 
+        if(!isLinear && get("init.type", env) == "coef"){
+            stop("You cannot initialize feglm with coefficients when there is no variable to be estimated (only the fixed-effects).")
+        }
+
     }
 
     # families of feglm
@@ -1533,7 +1601,7 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
             y_pos = lhs[lhs > 0]
             qui_pos = lhs > 0
             if(isWeight){
-                constant = sum(weights[qui_pos] * y_pos * cpppar_log(y_pos, nthreads) - weights[qui_pos] * y_pos)
+                constant = sum(weights.value[qui_pos] * y_pos * cpppar_log(y_pos, nthreads) - weights.value[qui_pos] * y_pos)
                 dev.resids = function(y, mu, eta, wt) 2 * (constant - sum(wt[qui_pos] * y_pos * eta[qui_pos]) + sum(wt * mu))
             } else {
                 constant = sum(y_pos * cpppar_log(y_pos, nthreads) - y_pos)
@@ -2024,6 +2092,18 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
 
         if(isSlope){
             res$fixef_terms = fixef_terms
+
+            # We also add the variables => makes fixef self-contained
+            # It looks odd but the user may ask for the same var to have
+            # varying slopes in svl dimensions
+            sv = list()
+            slope_vars_unik = slope_vars[!is.na(slope_vars)]
+            if(IS_REORDER) slope_variables = slope_variables[order(new_order)]
+            for(var in slope_vars_unik){
+                sv[[var]] = slope_variables[[which.max(slope_vars == var)]]
+            }
+
+            res$slope_variables = sv
         }
 
         res$fixef_id = fixef_id_res
@@ -2047,6 +2127,24 @@ fixest_env <- function(fml, data, family=c("poisson", "negbin", "logit", "gaussi
         res$weights = weights.value
     }
 
+    # We save lhs in case of feglm, for later within-r2 evaluation
+    if(origin_type == "feglm" && isFixef){
+        res$y = lhs
+    }
+
+    # Panel information
+    if(isPanel){
+        res$panel.id = panel.id
+        if(!is.null(panel.info)){
+            res$panel.info = panel.info
+        }
+    }
+
+    # Interaction information
+    if(!is.null(interaction.info)){
+        res$interaction.info = interaction.info
+    }
+
     assign("res", res, env)
 
     env
