@@ -2,12 +2,14 @@
 
 
 .onLoad <- function(libname, pkgname){
-	# setting the two options
+	# setting some options
 
 	options("fixest_dict" = c())
 	options("fixest_notes" = TRUE)
 	options("fixest_na_inf.rm" = TRUE)
 	options("fixest_print.type" = "table")
+	options("fixest_fl_authorized" = FALSE)
+	setFixest_dof()
 	setFixest_nthreads()
 
 	invisible()
